# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from .DeployServiceCommon_pb2 import (
    BlockInfo as DeployServiceCommon_pb2___BlockInfo,
    ContinuationsWithBlockInfo as DeployServiceCommon_pb2___ContinuationsWithBlockInfo,
    DataWithBlockInfo as DeployServiceCommon_pb2___DataWithBlockInfo,
    LightBlockInfo as DeployServiceCommon_pb2___LightBlockInfo,
)

from .ServiceError_pb2 import (
    ServiceError as ServiceError_pb2___ServiceError,
)

from google.protobuf.descriptor import (
    Descriptor as google___protobuf___descriptor___Descriptor,
)

from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Iterable as typing___Iterable,
    Optional as typing___Optional,
    Text as typing___Text,
    Union as typing___Union,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


builtin___bool = bool
builtin___bytes = bytes
builtin___float = float
builtin___int = int
if sys.version_info < (3,):
    builtin___buffer = buffer
    builtin___unicode = unicode


class ExploratoryDeployResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    @property
    def result(self) -> DeployServiceCommon_pb2___DataWithBlockInfo: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        result : typing___Optional[DeployServiceCommon_pb2___DataWithBlockInfo] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ExploratoryDeployResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ExploratoryDeployResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"error",b"error",u"message",b"message",u"result",b"result"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"error",b"error",u"message",b"message",u"result",b"result"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","result"]: ...

class DeployResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    result = ... # type: typing___Text

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        result : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> DeployResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> DeployResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"error",b"error",u"message",b"message",u"result",b"result"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"error",b"error",u"message",b"message",u"result",b"result"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","result"]: ...

class BlockResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    @property
    def blockInfo(self) -> DeployServiceCommon_pb2___BlockInfo: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        blockInfo : typing___Optional[DeployServiceCommon_pb2___BlockInfo] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BlockResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BlockResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"blockInfo",b"blockInfo",u"error",b"error",u"message",b"message"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockInfo",b"blockInfo",u"error",b"error",u"message",b"message"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","blockInfo"]: ...

class VisualizeBlocksResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    content = ... # type: typing___Text

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        content : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> VisualizeBlocksResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> VisualizeBlocksResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"content",b"content",u"error",b"error",u"message",b"message"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"content",b"content",u"error",b"error",u"message",b"message"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","content"]: ...

class MachineVerifyResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    content = ... # type: typing___Text

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        content : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> MachineVerifyResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> MachineVerifyResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"content",b"content",u"error",b"error",u"message",b"message"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"content",b"content",u"error",b"error",u"message",b"message"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","content"]: ...

class BlockInfoResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    @property
    def blockInfo(self) -> DeployServiceCommon_pb2___LightBlockInfo: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        blockInfo : typing___Optional[DeployServiceCommon_pb2___LightBlockInfo] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BlockInfoResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BlockInfoResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"blockInfo",b"blockInfo",u"error",b"error",u"message",b"message"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockInfo",b"blockInfo",u"error",b"error",u"message",b"message"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","blockInfo"]: ...

class ListeningNameDataResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    @property
    def payload(self) -> ListeningNameDataPayload: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        payload : typing___Optional[ListeningNameDataPayload] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ListeningNameDataResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ListeningNameDataResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"error",b"error",u"message",b"message",u"payload",b"payload"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"error",b"error",u"message",b"message",u"payload",b"payload"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","payload"]: ...

class ListeningNameDataPayload(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    length = ... # type: builtin___int

    @property
    def blockInfo(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[DeployServiceCommon_pb2___DataWithBlockInfo]: ...

    def __init__(self,
        *,
        blockInfo : typing___Optional[typing___Iterable[DeployServiceCommon_pb2___DataWithBlockInfo]] = None,
        length : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ListeningNameDataPayload: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ListeningNameDataPayload: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockInfo",b"blockInfo",u"length",b"length"]) -> None: ...

class ContinuationAtNameResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    @property
    def payload(self) -> ContinuationAtNamePayload: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        payload : typing___Optional[ContinuationAtNamePayload] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ContinuationAtNameResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ContinuationAtNameResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"error",b"error",u"message",b"message",u"payload",b"payload"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"error",b"error",u"message",b"message",u"payload",b"payload"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","payload"]: ...

class ContinuationAtNamePayload(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    length = ... # type: builtin___int

    @property
    def blockResults(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[DeployServiceCommon_pb2___ContinuationsWithBlockInfo]: ...

    def __init__(self,
        *,
        blockResults : typing___Optional[typing___Iterable[DeployServiceCommon_pb2___ContinuationsWithBlockInfo]] = None,
        length : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ContinuationAtNamePayload: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ContinuationAtNamePayload: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockResults",b"blockResults",u"length",b"length"]) -> None: ...

class FindDeployResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    @property
    def blockInfo(self) -> DeployServiceCommon_pb2___LightBlockInfo: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        blockInfo : typing___Optional[DeployServiceCommon_pb2___LightBlockInfo] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> FindDeployResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> FindDeployResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"blockInfo",b"blockInfo",u"error",b"error",u"message",b"message"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockInfo",b"blockInfo",u"error",b"error",u"message",b"message"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","blockInfo"]: ...

class PrivateNamePreviewResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    @property
    def payload(self) -> PrivateNamePreviewPayload: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        payload : typing___Optional[PrivateNamePreviewPayload] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> PrivateNamePreviewResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> PrivateNamePreviewResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"error",b"error",u"message",b"message",u"payload",b"payload"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"error",b"error",u"message",b"message",u"payload",b"payload"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","payload"]: ...

class PrivateNamePreviewPayload(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    ids = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[builtin___bytes]

    def __init__(self,
        *,
        ids : typing___Optional[typing___Iterable[builtin___bytes]] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> PrivateNamePreviewPayload: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> PrivateNamePreviewPayload: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"ids",b"ids"]) -> None: ...

class LastFinalizedBlockResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    @property
    def blockInfo(self) -> DeployServiceCommon_pb2___BlockInfo: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        blockInfo : typing___Optional[DeployServiceCommon_pb2___BlockInfo] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> LastFinalizedBlockResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> LastFinalizedBlockResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"blockInfo",b"blockInfo",u"error",b"error",u"message",b"message"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockInfo",b"blockInfo",u"error",b"error",u"message",b"message"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","blockInfo"]: ...

class IsFinalizedResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    isFinalized = ... # type: builtin___bool

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        isFinalized : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> IsFinalizedResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> IsFinalizedResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"error",b"error",u"isFinalized",b"isFinalized",u"message",b"message"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"error",b"error",u"isFinalized",b"isFinalized",u"message",b"message"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","isFinalized"]: ...

class BondStatusResponse(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    isBonded = ... # type: builtin___bool

    @property
    def error(self) -> ServiceError_pb2___ServiceError: ...

    def __init__(self,
        *,
        error : typing___Optional[ServiceError_pb2___ServiceError] = None,
        isBonded : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BondStatusResponse: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BondStatusResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"error",b"error",u"isBonded",b"isBonded",u"message",b"message"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"error",b"error",u"isBonded",b"isBonded",u"message",b"message"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"message",b"message"]) -> typing_extensions___Literal["error","isBonded"]: ...



import random
import string
from dataclasses import dataclass, field
from typing import List
from unittest.mock import Mock

from ..meta import from_pb

try:
    # python3.6 doesn't have it
    from unittest.mock import seal
except ImportError:
    def seal(obj):
        return obj



@from_pb
@dataclass
class Class1:
    value1: str
    value2: int
    value3: List[str]
    value4: float


@from_pb
@dataclass
class ClassEmbedded:
    value2: str
    value1: Class1 = field(default_factory=Class1)


@from_pb
@dataclass
class ClassEmbeddedList:
    value1: str
    value2: int
    value3: List[str]
    value4: float
    value5: List[Class1] = field(default_factory=Class1)


def generate_Class1_pb():
    obj = Mock()
    obj.value1 = random.choice(string.ascii_lowercase)
    obj.value2 = random.randint(0, 1000)
    obj.value3 = [random.choice(string.ascii_lowercase) for _ in range(5)]
    obj.value4 = random.randint(1, 10000) / 10.0
    seal(obj)
    return obj


def generate_ClassEmbedded_pb():
    obj = Mock()
    obj.value1 = generate_Class1_pb()
    obj.value2 = random.choice(string.ascii_lowercase)
    seal(obj)
    return obj


def generate_ClassEmbeddedList_pb():
    obj = Mock()
    obj.value1 = random.choice(string.ascii_lowercase)
    obj.value2 = random.randint(0, 1000)
    obj.value3 = [random.choice(string.ascii_lowercase) for _ in range(5)]
    obj.value4 = random.randint(1, 10000) / 10.0
    obj.value5 = [generate_Class1_pb() for _ in range(2)]
    seal(obj)
    return obj


def test_simple_gen_from_pb():
    c1_pb = generate_Class1_pb()
    c1 = Class1.from_pb(c1_pb)
    assert c1.value1 == c1_pb.value1
    assert c1.value2 == c1_pb.value2
    assert c1.value3 == c1_pb.value3
    assert c1.value4 == c1_pb.value4


def test_gen_from_pb_embedded():
    embedded_pb = generate_ClassEmbedded_pb()
    c = ClassEmbedded.from_pb(embedded_pb)
    assert c.value1.value1 == embedded_pb.value1.value1
    assert c.value1.value2 == embedded_pb.value1.value2
    assert c.value1.value3 == embedded_pb.value1.value3
    assert c.value1.value4 == embedded_pb.value1.value4
    assert c.value2 == embedded_pb.value2


def test_gen_from_pb_embedded_list():
    embedded_pb = generate_ClassEmbeddedList_pb()
    embedded_obj = ClassEmbeddedList.from_pb(embedded_pb)
    assert embedded_obj.value1 == embedded_pb.value1
    assert embedded_obj.value2 == embedded_pb.value2
    assert embedded_obj.value3 == embedded_pb.value3
    assert embedded_obj.value4 == embedded_pb.value4
    for i in range(len(embedded_pb.value5)):
        assert embedded_obj.value5[i].value1 == embedded_pb.value5[i].value1
        assert embedded_obj.value5[i].value2 == embedded_pb.value5[i].value2
        assert embedded_obj.value5[i].value3 == embedded_pb.value5[i].value3
        assert embedded_obj.value5[i].value4 == embedded_pb.value5[i].value4



from abc import ABC

from .crypto import PrivateKey

# from .data_types import BlockInfo, LightBlockInfo


class APIClient(ABC):
    def deploy(self, key: PrivateKey,
               term: str,
               phlo_price: int,
               phlo_limit: int,
               valid_after_block_no: int,
               timestamp_millis: int) -> str:
        raise NotImplementedError()

    def deploy_with_vabn_filled(self, key: PrivateKey,
                                term: str,
                                phlo_price: int,
                                phlo_limit: int,
                                timestamp_millis: int) -> str:
        raise NotImplementedError()

    def show_block(self, block_hash: str):
        raise NotImplementedError()

    def show_blocks(self, depth: int):
        raise NotImplementedError()

    def propose(self) -> str:
        raise NotImplementedError()

    def listen_for_data_at_name(self):
        raise NotImplementedError()



import string
import time
from typing import Mapping

from .client import RClient
from .crypto import PrivateKey

CREATE_VAULT_RHO_TPL = """
new rl(`rho:registry:lookup`), RevVaultCh in {
  rl!(`rho:rchain:revVault`, *RevVaultCh) |
  for (@(_, RevVault) <- RevVaultCh) {
    @RevVault!("findOrCreateVault", "$addr", Nil)
  }
}
"""

GET_BALANCE_RHO_TPL = """
new return, rl(`rho:registry:lookup`), RevVaultCh, vaultCh, balanceCh in {
  rl!(`rho:rchain:revVault`, *RevVaultCh) |
  for (@(_, RevVault) <- RevVaultCh) {
    @RevVault!("findOrCreate", "$addr", *vaultCh) |
    for (@(true, vault) <- vaultCh) {
      @vault!("balance", *balanceCh) |
      for (@balance <- balanceCh) {
        return!(balance)
      }
    }
  }
}
"""

TRANSFER_RHO_TPL = """
new rl(`rho:registry:lookup`), RevVaultCh, vaultCh, revVaultKeyCh, resultCh in {
  rl!(`rho:rchain:revVault`, *RevVaultCh) |
  for (@(_, RevVault) <- RevVaultCh) {
    @RevVault!("findOrCreate", "$from", *vaultCh) |
    @RevVault!("deployerAuthKey", *revVaultKeyCh) |
    for (@(true, vault) <- vaultCh; key <- revVaultKeyCh) {
      @vault!("transfer", "$to", $amount, *key, *resultCh) |
      for (_ <- resultCh) { Nil }
    }
  }
}
"""

TRANSFER_ENSURE_TO_RHO_TPL = """
new rl(`rho:registry:lookup`), RevVaultCh, vaultCh, toVaultCh, revVaultKeyCh, resultCh in {
  rl!(`rho:rchain:revVault`, *RevVaultCh) |
  for (@(_, RevVault) <- RevVaultCh) {
    @RevVault!("findOrCreate", "$from", *vaultCh) |
    @RevVault!("findOrCreate", "$to", *toVaultCh) |
    @RevVault!("deployerAuthKey", *revVaultKeyCh) |
    for (@(true, vault) <- vaultCh; key <- revVaultKeyCh; @(true, toVault) <- toVaultCh;) {
      @vault!("transfer", "$to", $amount, *key, *resultCh) |
      for (_ <- resultCh) { Nil }
    }
  }
}
"""

# these are predefined param
TRANSFER_PHLO_LIMIT = 100000
TRANSFER_PHLO_PRICE = 1


def render_contract_template(template: str, substitutions: Mapping[str, str]) -> str:
    return string.Template(template).substitute(substitutions)


class VaultAPI:

    def __init__(self, client: RClient):
        self.client = client

    def get_balance(self, rev_addr: str) -> int:
        contract = render_contract_template(
            GET_BALANCE_RHO_TPL,
            {'addr': rev_addr},
        )
        result = self.client.exploratory_deploy(contract)
        return int(result[0].exprs[0].g_int)

    def transfer(self, from_addr: str, to_addr: str, amount: int, key: PrivateKey) -> str:
        """
        Transfer from `from_addr` to `to_addr` in the chain. Just make sure the `to_addr` is created
        in the chain. Otherwise, the transfer would hang until the `to_addr` is created.
        """
        contract = render_contract_template(
            TRANSFER_RHO_TPL, {
                'from': from_addr,
                'to': to_addr,
                'amount': str(amount)
            }
        )
        timestamp_mill = int(time.time() * 1000)
        return self.client.deploy_with_vabn_filled(key, contract, TRANSFER_PHLO_PRICE, TRANSFER_PHLO_LIMIT,
                                                   timestamp_mill)

    def transfer_ensure(self, from_addr: str, to_addr: str, amount: int, key: PrivateKey) -> str:
        """
        The difference between `transfer_ensure` and `transfer` is that , if the to_addr is not created in the
        chain, the `transfer` would hang until the to_addr successfully created in the change and the `transfer_ensure`
        can be sure that if the `to_addr` is not existed in the chain the process would created the vault in the chain
        and make the transfer successfully.
        """
        contract = render_contract_template(
            TRANSFER_ENSURE_TO_RHO_TPL, {
                'from': from_addr,
                'to': to_addr,
                'amount': str(amount)
            }
        )
        timestamp_mill = int(time.time()* 1000)
        return self.client.deploy_with_vabn_filled(key, contract, TRANSFER_PHLO_PRICE, TRANSFER_PHLO_LIMIT,
                                                   timestamp_mill)

    def create_vault(self, addr: str, key: PrivateKey) -> str:
        contract = render_contract_template(
            CREATE_VAULT_RHO_TPL, {
                'addr': addr
            }
        )
        timestamp_mill = int(time.time() * 1000)
        return self.client.deploy_with_vabn_filled(key, contract, TRANSFER_PHLO_PRICE, TRANSFER_PHLO_LIMIT, timestamp_millis=timestamp_mill)


from concurrent import futures
from contextlib import contextmanager
from typing import Generator, Tuple, Union

import grpc
import pytest
from rchain.client import RClient
from rchain.crypto import PrivateKey
from rchain.pb.CasperMessage_pb2 import DeployDataProto
from rchain.pb.DeployServiceCommon_pb2 import (
    BlockInfo, BlockQuery, BlocksQuery, BondInfo, DeployInfo, FindDeployQuery,
    IsFinalizedQuery, LastFinalizedBlockQuery, LightBlockInfo,
)
from rchain.pb.DeployServiceV1_pb2 import (
    BlockInfoResponse, BlockResponse, DeployResponse, FindDeployResponse,
    IsFinalizedResponse, LastFinalizedBlockResponse,
)
from rchain.pb.DeployServiceV1_pb2_grpc import (
    DeployServiceServicer, add_DeployServiceServicer_to_server,
)
from rchain.pb.ProposeServiceCommon_pb2 import PrintUnmatchedSendsQuery
from rchain.pb.ProposeServiceV1_pb2 import ProposeResponse
from rchain.pb.ProposeServiceV1_pb2_grpc import (
    ProposeServiceServicer, add_ProposeServiceServicer_to_server,
)
from rchain.util import create_deploy_data, verify_deploy_data

key = PrivateKey.generate()


@contextmanager
def deploy_service(deploy_service: Union[DeployServiceServicer, ProposeServiceServicer]) -> Generator[
    Tuple[grpc.Server, int], None, None]:
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=2))
    if isinstance(deploy_service, DeployServiceServicer):
        add_DeployServiceServicer_to_server(deploy_service, server)
    if isinstance(deploy_service, ProposeServiceServicer):
        add_ProposeServiceServicer_to_server(deploy_service, server)
    port = server.add_insecure_port("0.0.0.0:9766")
    assert port != 0
    server.start()
    yield server, port
    server.stop(0)


TEST_HOST = '127.0.0.1'


@pytest.mark.parametrize("key,terms,phlo_price,phlo_limit,valid_after_block_no,timestamp_millis", [
    (key, "@0!(2)", 1, 10000, 1, 1000),
    (key, "@0!(2) | @1!(1)", 1, 10000, 10, 1000),
    (key, "@0!(2)", 10, 200000, 10, 3000),
])
def test_client_deploy(key: PrivateKey, terms: str, phlo_price: int, phlo_limit: int, valid_after_block_no: int,
                       timestamp_millis: int) -> None:
    class DummyDeploySerivce(DeployServiceServicer):
        def doDeploy(self, request: DeployDataProto, context: grpc.ServicerContext) -> DeployResponse:
            return DeployResponse(result=request.sig.hex())

    with deploy_service(DummyDeploySerivce()) as (server, port), \
            RClient(TEST_HOST, port) as client:
        ret = client.deploy(key, terms, phlo_price, phlo_limit, valid_after_block_no, timestamp_millis)
        assert verify_deploy_data(key.get_public_key(), bytes.fromhex(ret),
                                  create_deploy_data(key, terms, phlo_price, phlo_limit, valid_after_block_no,
                                                     timestamp_millis))


def test_client_show_block() -> None:
    request_block_hash = "asdasdasdasd"
    sender = "a"
    seqNum = 1
    sig = "sig"
    sigAlgorithm = "sigal"
    shardId = "rchain"
    extraBytes = b"extraBytes"
    version = 1
    timestamp = 10000000
    headerExtraBytes = b"headerExtraBytes"
    parentsHashList = ["abc"]
    blockNumber = 1
    preStateHash = "preStateHash"
    postStateHash = "postStateHash"
    bodyExtraBytes = b"bodyExtraBytes"
    bond = BondInfo(validator="a", stake=100)
    blockSize = "100"
    deployCount = 1
    faultTolerance = 0.2
    deploy = DeployInfo(
        deployer="a",
        term="1",
        timestamp=timestamp,
        sig=sig,
        sigAlgorithm=sigAlgorithm,
        phloPrice=1,
        phloLimit=100000,
        validAfterBlockNumber=1,
        cost=100,
        errored=False,
        systemDeployError="none"

    )

    class DummyDeploySerivce(DeployServiceServicer):
        def getBlock(self, request: BlockQuery, context: grpc.ServicerContext) -> BlockResponse:
            return BlockResponse(blockInfo=BlockInfo(blockInfo=LightBlockInfo(
                blockHash=request_block_hash,
                sender=sender, seqNum=seqNum, sig=sig, sigAlgorithm=sigAlgorithm, shardId=shardId,
                extraBytes=extraBytes, version=version, timestamp=timestamp, headerExtraBytes=headerExtraBytes,
                parentsHashList=parentsHashList, blockNumber=blockNumber, preStateHash=preStateHash,
                postStateHash=postStateHash, bodyExtraBytes=bodyExtraBytes, bonds=[bond], blockSize=blockSize,
                deployCount=deployCount, faultTolerance=faultTolerance
            ), deploys=[deploy]))

    with deploy_service(DummyDeploySerivce()) as (server, port), \
            RClient(TEST_HOST, port) as client:
        block = client.show_block(request_block_hash)
        block_info = block.blockInfo
        assert block_info.blockHash == request_block_hash
        assert block_info.sender == sender
        assert block_info.seqNum == seqNum
        assert block_info.sig == sig
        assert block_info.sigAlgorithm == sigAlgorithm
        assert block_info.shardId == shardId
        assert block_info.extraBytes == extraBytes
        assert block_info.version == version
        assert block_info.timestamp == timestamp
        assert block_info.headerExtraBytes == headerExtraBytes
        assert block_info.parentsHashList == parentsHashList
        assert block_info.blockNumber == blockNumber
        assert block_info.preStateHash == preStateHash
        assert block_info.postStateHash == postStateHash
        assert block_info.bodyExtraBytes == bodyExtraBytes
        assert block_info.blockSize == blockSize
        assert block_info.deployCount == deployCount
        assert pytest.approx(block_info.faultTolerance, faultTolerance)
        bond_info = block_info.bonds[0]
        assert bond_info.validator == bond.validator
        assert bond_info.stake == bond.stake

        deploy_info = block.deploys[0]
        assert deploy_info.deployer == deploy.deployer
        assert deploy_info.term == deploy.term
        assert deploy_info.timestamp == deploy.timestamp
        assert deploy_info.sig == deploy.sig
        assert deploy_info.sigAlgorithm == deploy.sigAlgorithm
        assert deploy_info.phloPrice == deploy.phloPrice
        assert deploy_info.phloLimit == deploy.phloLimit
        assert deploy_info.validAfterBlockNumber == deploy.validAfterBlockNumber
        assert deploy_info.cost == deploy.cost
        assert deploy_info.errored == deploy.errored
        assert deploy_info.systemDeployError == deploy.systemDeployError


def test_client_show_blocks() -> None:
    request_block_hash = "asdasdasdasd"
    sender = "a"
    seqNum = 1
    sig = "sig"
    sigAlgorithm = "sigal"
    shardId = "rchain"
    extraBytes = b"extraBytes"
    version = 1
    timestamp = 10000000
    headerExtraBytes = b"headerExtraBytes"
    parentsHashList = ["abc"]
    blockNumber = 1
    preStateHash = "preStateHash"
    postStateHash = "postStateHash"
    bodyExtraBytes = b"bodyExtraBytes"
    bond = BondInfo(validator="a", stake=100)
    blockSize = "100"
    deployCount = 1
    faultTolerance = 0.2

    class DummyDeploySerivce(DeployServiceServicer):
        def getBlocks(self, request: BlocksQuery, context: grpc.ServicerContext) -> Generator[
            BlockInfoResponse, None, None]:
            yield BlockInfoResponse(blockInfo=LightBlockInfo(
                blockHash=request_block_hash,
                sender=sender, seqNum=seqNum, sig=sig, sigAlgorithm=sigAlgorithm, shardId=shardId,
                extraBytes=extraBytes, version=version, timestamp=timestamp, headerExtraBytes=headerExtraBytes,
                parentsHashList=parentsHashList, blockNumber=blockNumber, preStateHash=preStateHash,
                postStateHash=postStateHash, bodyExtraBytes=bodyExtraBytes, bonds=[bond], blockSize=blockSize,
                deployCount=deployCount, faultTolerance=faultTolerance))

    with deploy_service(DummyDeploySerivce()) as (server, port), \
            RClient(TEST_HOST, port) as client:
        blocks = client.show_blocks()
        block_info = blocks[0]
        assert len(blocks) == 1
        assert block_info.blockHash == request_block_hash
        assert block_info.sender == sender
        assert block_info.seqNum == seqNum
        assert block_info.sig == sig
        assert block_info.sigAlgorithm == sigAlgorithm
        assert block_info.shardId == shardId
        assert block_info.extraBytes == extraBytes
        assert block_info.version == version
        assert block_info.timestamp == timestamp
        assert block_info.headerExtraBytes == headerExtraBytes
        assert block_info.parentsHashList == parentsHashList
        assert block_info.blockNumber == blockNumber
        assert block_info.preStateHash == preStateHash
        assert block_info.postStateHash == postStateHash
        assert block_info.bodyExtraBytes == bodyExtraBytes
        assert block_info.blockSize == blockSize
        assert block_info.deployCount == deployCount
        assert pytest.approx(block_info.faultTolerance, faultTolerance)
        bond_info = block_info.bonds[0]
        assert bond_info.validator == bond.validator
        assert bond_info.stake == bond.stake


def test_client_propose() -> None:
    block_hash = "abcabcabc"

    class DummyProposeService(ProposeServiceServicer):
        def propose(self, request: PrintUnmatchedSendsQuery, context: grpc.ServicerContext) -> ProposeResponse:
            return ProposeResponse(result="Success! Block {} created and added.".format(block_hash))

    with deploy_service(DummyProposeService()) as (server, port), \
            RClient(TEST_HOST, port) as client:
        hash = client.propose()
        assert hash == block_hash


def test_client_find_deploy() -> None:
    deploy_id = '61e594124ca6af84a5468d98b34a4f3431ef39c54c6cf07fe6fbf8b079ef64f6'

    class DummyDeployService(DeployServiceServicer):
        def findDeploy(self, request: FindDeployQuery, context: grpc.ServicerContext) -> FindDeployResponse:
            return FindDeployResponse(blockInfo=LightBlockInfo())

    with deploy_service(DummyDeployService()) as (server, port), \
            RClient(TEST_HOST, port) as client:
        assert client.find_deploy(deploy_id)


def test_client_last_finalized_block() -> None:
    class DummyDeployService(DeployServiceServicer):
        def lastFinalizedBlock(self, request: LastFinalizedBlockQuery,
                               context: grpc.ServicerContext) -> LastFinalizedBlockResponse:
            return LastFinalizedBlockResponse(blockInfo=BlockInfo())

    with deploy_service(DummyDeployService()) as (server, port), \
            RClient(TEST_HOST, port) as client:
        assert client.last_finalized_block()


def test_client_is_finalized_block() -> None:
    class DummyDeployService(DeployServiceServicer):
        def isFinalized(self, request: IsFinalizedQuery, context: grpc.ServicerContext) -> IsFinalizedResponse:
            return IsFinalizedResponse(isFinalized=True)

    with deploy_service(DummyDeployService()) as (server, port), \
            RClient(TEST_HOST, port) as client:
        assert client.is_finalized('asd')



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from .RhoTypes_pb2 import (
    PCost as RhoTypes_pb2___PCost,
)

from google.protobuf.descriptor import (
    Descriptor as google___protobuf___descriptor___Descriptor,
)

from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Iterable as typing___Iterable,
    Optional as typing___Optional,
    Text as typing___Text,
    Union as typing___Union,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


builtin___bool = bool
builtin___bytes = bytes
builtin___float = float
builtin___int = int
if sys.version_info < (3,):
    builtin___buffer = buffer
    builtin___unicode = unicode


class HasBlockRequestProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    hash = ... # type: builtin___bytes

    def __init__(self,
        *,
        hash : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> HasBlockRequestProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> HasBlockRequestProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"hash",b"hash"]) -> None: ...

class HasBlockProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    hash = ... # type: builtin___bytes

    def __init__(self,
        *,
        hash : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> HasBlockProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> HasBlockProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"hash",b"hash"]) -> None: ...

class BlockRequestProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    hash = ... # type: builtin___bytes

    def __init__(self,
        *,
        hash : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BlockRequestProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BlockRequestProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"hash",b"hash"]) -> None: ...

class ForkChoiceTipRequestProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    def __init__(self,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ForkChoiceTipRequestProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ForkChoiceTipRequestProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class ApprovedBlockCandidateProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    requiredSigs = ... # type: builtin___int

    @property
    def block(self) -> BlockMessageProto: ...

    def __init__(self,
        *,
        block : typing___Optional[BlockMessageProto] = None,
        requiredSigs : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ApprovedBlockCandidateProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ApprovedBlockCandidateProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"block",b"block"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"block",b"block",u"requiredSigs",b"requiredSigs"]) -> None: ...

class UnapprovedBlockProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    timestamp = ... # type: builtin___int
    duration = ... # type: builtin___int

    @property
    def candidate(self) -> ApprovedBlockCandidateProto: ...

    def __init__(self,
        *,
        candidate : typing___Optional[ApprovedBlockCandidateProto] = None,
        timestamp : typing___Optional[builtin___int] = None,
        duration : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> UnapprovedBlockProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> UnapprovedBlockProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"candidate",b"candidate"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"candidate",b"candidate",u"duration",b"duration",u"timestamp",b"timestamp"]) -> None: ...

class Signature(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    publicKey = ... # type: builtin___bytes
    algorithm = ... # type: typing___Text
    sig = ... # type: builtin___bytes

    def __init__(self,
        *,
        publicKey : typing___Optional[builtin___bytes] = None,
        algorithm : typing___Optional[typing___Text] = None,
        sig : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Signature: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Signature: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"algorithm",b"algorithm",u"publicKey",b"publicKey",u"sig",b"sig"]) -> None: ...

class BlockApprovalProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def candidate(self) -> ApprovedBlockCandidateProto: ...

    @property
    def sig(self) -> Signature: ...

    def __init__(self,
        *,
        candidate : typing___Optional[ApprovedBlockCandidateProto] = None,
        sig : typing___Optional[Signature] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BlockApprovalProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BlockApprovalProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"candidate",b"candidate",u"sig",b"sig"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"candidate",b"candidate",u"sig",b"sig"]) -> None: ...

class ApprovedBlockProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def candidate(self) -> ApprovedBlockCandidateProto: ...

    @property
    def sigs(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Signature]: ...

    def __init__(self,
        *,
        candidate : typing___Optional[ApprovedBlockCandidateProto] = None,
        sigs : typing___Optional[typing___Iterable[Signature]] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ApprovedBlockProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ApprovedBlockProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"candidate",b"candidate"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"candidate",b"candidate",u"sigs",b"sigs"]) -> None: ...

class ApprovedBlockRequestProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    identifier = ... # type: typing___Text

    def __init__(self,
        *,
        identifier : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ApprovedBlockRequestProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ApprovedBlockRequestProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"identifier",b"identifier"]) -> None: ...

class NoApprovedBlockAvailableProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    identifier = ... # type: typing___Text
    nodeIdentifer = ... # type: typing___Text

    def __init__(self,
        *,
        identifier : typing___Optional[typing___Text] = None,
        nodeIdentifer : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> NoApprovedBlockAvailableProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> NoApprovedBlockAvailableProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"identifier",b"identifier",u"nodeIdentifer",b"nodeIdentifer"]) -> None: ...

class BlockMessageProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    blockHash = ... # type: builtin___bytes
    sender = ... # type: builtin___bytes
    seqNum = ... # type: builtin___int
    sig = ... # type: builtin___bytes
    sigAlgorithm = ... # type: typing___Text
    shardId = ... # type: typing___Text
    extraBytes = ... # type: builtin___bytes

    @property
    def header(self) -> HeaderProto: ...

    @property
    def body(self) -> BodyProto: ...

    @property
    def justifications(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[JustificationProto]: ...

    def __init__(self,
        *,
        blockHash : typing___Optional[builtin___bytes] = None,
        header : typing___Optional[HeaderProto] = None,
        body : typing___Optional[BodyProto] = None,
        justifications : typing___Optional[typing___Iterable[JustificationProto]] = None,
        sender : typing___Optional[builtin___bytes] = None,
        seqNum : typing___Optional[builtin___int] = None,
        sig : typing___Optional[builtin___bytes] = None,
        sigAlgorithm : typing___Optional[typing___Text] = None,
        shardId : typing___Optional[typing___Text] = None,
        extraBytes : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BlockMessageProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BlockMessageProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"body",b"body",u"header",b"header"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockHash",b"blockHash",u"body",b"body",u"extraBytes",b"extraBytes",u"header",b"header",u"justifications",b"justifications",u"sender",b"sender",u"seqNum",b"seqNum",u"shardId",b"shardId",u"sig",b"sig",u"sigAlgorithm",b"sigAlgorithm"]) -> None: ...

class BlockHashMessageProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    hash = ... # type: builtin___bytes
    blockCreator = ... # type: builtin___bytes

    def __init__(self,
        *,
        hash : typing___Optional[builtin___bytes] = None,
        blockCreator : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BlockHashMessageProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BlockHashMessageProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockCreator",b"blockCreator",u"hash",b"hash"]) -> None: ...

class BlockMetadataInternal(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    blockHash = ... # type: builtin___bytes
    parents = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[builtin___bytes]
    sender = ... # type: builtin___bytes
    blockNum = ... # type: builtin___int
    seqNum = ... # type: builtin___int
    invalid = ... # type: builtin___bool

    @property
    def justifications(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[JustificationProto]: ...

    @property
    def bonds(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[BondProto]: ...

    def __init__(self,
        *,
        blockHash : typing___Optional[builtin___bytes] = None,
        parents : typing___Optional[typing___Iterable[builtin___bytes]] = None,
        sender : typing___Optional[builtin___bytes] = None,
        justifications : typing___Optional[typing___Iterable[JustificationProto]] = None,
        bonds : typing___Optional[typing___Iterable[BondProto]] = None,
        blockNum : typing___Optional[builtin___int] = None,
        seqNum : typing___Optional[builtin___int] = None,
        invalid : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BlockMetadataInternal: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BlockMetadataInternal: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockHash",b"blockHash",u"blockNum",b"blockNum",u"bonds",b"bonds",u"invalid",b"invalid",u"justifications",b"justifications",u"parents",b"parents",u"sender",b"sender",u"seqNum",b"seqNum"]) -> None: ...

class HeaderProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    parentsHashList = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[builtin___bytes]
    timestamp = ... # type: builtin___int
    version = ... # type: builtin___int
    extraBytes = ... # type: builtin___bytes

    def __init__(self,
        *,
        parentsHashList : typing___Optional[typing___Iterable[builtin___bytes]] = None,
        timestamp : typing___Optional[builtin___int] = None,
        version : typing___Optional[builtin___int] = None,
        extraBytes : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> HeaderProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> HeaderProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"extraBytes",b"extraBytes",u"parentsHashList",b"parentsHashList",u"timestamp",b"timestamp",u"version",b"version"]) -> None: ...

class DeployDataProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    deployer = ... # type: builtin___bytes
    term = ... # type: typing___Text
    timestamp = ... # type: builtin___int
    sig = ... # type: builtin___bytes
    sigAlgorithm = ... # type: typing___Text
    phloPrice = ... # type: builtin___int
    phloLimit = ... # type: builtin___int
    validAfterBlockNumber = ... # type: builtin___int

    def __init__(self,
        *,
        deployer : typing___Optional[builtin___bytes] = None,
        term : typing___Optional[typing___Text] = None,
        timestamp : typing___Optional[builtin___int] = None,
        sig : typing___Optional[builtin___bytes] = None,
        sigAlgorithm : typing___Optional[typing___Text] = None,
        phloPrice : typing___Optional[builtin___int] = None,
        phloLimit : typing___Optional[builtin___int] = None,
        validAfterBlockNumber : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> DeployDataProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> DeployDataProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"deployer",b"deployer",u"phloLimit",b"phloLimit",u"phloPrice",b"phloPrice",u"sig",b"sig",u"sigAlgorithm",b"sigAlgorithm",u"term",b"term",u"timestamp",b"timestamp",u"validAfterBlockNumber",b"validAfterBlockNumber"]) -> None: ...

class ProcessedDeployProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    errored = ... # type: builtin___bool
    systemDeployError = ... # type: typing___Text

    @property
    def deploy(self) -> DeployDataProto: ...

    @property
    def cost(self) -> RhoTypes_pb2___PCost: ...

    @property
    def deployLog(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[EventProto]: ...

    def __init__(self,
        *,
        deploy : typing___Optional[DeployDataProto] = None,
        cost : typing___Optional[RhoTypes_pb2___PCost] = None,
        deployLog : typing___Optional[typing___Iterable[EventProto]] = None,
        errored : typing___Optional[builtin___bool] = None,
        systemDeployError : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ProcessedDeployProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ProcessedDeployProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"cost",b"cost",u"deploy",b"deploy"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"cost",b"cost",u"deploy",b"deploy",u"deployLog",b"deployLog",u"errored",b"errored",u"systemDeployError",b"systemDeployError"]) -> None: ...

class SlashSystemDeployDataProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    invalidBlockHash = ... # type: builtin___bytes
    issuerPublicKey = ... # type: builtin___bytes

    def __init__(self,
        *,
        invalidBlockHash : typing___Optional[builtin___bytes] = None,
        issuerPublicKey : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> SlashSystemDeployDataProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> SlashSystemDeployDataProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"invalidBlockHash",b"invalidBlockHash",u"issuerPublicKey",b"issuerPublicKey"]) -> None: ...

class CloseBlockSystemDeployDataProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    def __init__(self,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> CloseBlockSystemDeployDataProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> CloseBlockSystemDeployDataProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class SystemDeployDataProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def slashSystemDeploy(self) -> SlashSystemDeployDataProto: ...

    @property
    def closeBlockSystemDeploy(self) -> CloseBlockSystemDeployDataProto: ...

    def __init__(self,
        *,
        slashSystemDeploy : typing___Optional[SlashSystemDeployDataProto] = None,
        closeBlockSystemDeploy : typing___Optional[CloseBlockSystemDeployDataProto] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> SystemDeployDataProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> SystemDeployDataProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"closeBlockSystemDeploy",b"closeBlockSystemDeploy",u"slashSystemDeploy",b"slashSystemDeploy",u"systemDeploy",b"systemDeploy"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"closeBlockSystemDeploy",b"closeBlockSystemDeploy",u"slashSystemDeploy",b"slashSystemDeploy",u"systemDeploy",b"systemDeploy"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"systemDeploy",b"systemDeploy"]) -> typing_extensions___Literal["slashSystemDeploy","closeBlockSystemDeploy"]: ...

class ProcessedSystemDeployProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    errorMsg = ... # type: typing___Text

    @property
    def systemDeploy(self) -> SystemDeployDataProto: ...

    @property
    def deployLog(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[EventProto]: ...

    def __init__(self,
        *,
        systemDeploy : typing___Optional[SystemDeployDataProto] = None,
        deployLog : typing___Optional[typing___Iterable[EventProto]] = None,
        errorMsg : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ProcessedSystemDeployProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ProcessedSystemDeployProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"systemDeploy",b"systemDeploy"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"deployLog",b"deployLog",u"errorMsg",b"errorMsg",u"systemDeploy",b"systemDeploy"]) -> None: ...

class BodyProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    extraBytes = ... # type: builtin___bytes

    @property
    def state(self) -> RChainStateProto: ...

    @property
    def deploys(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[ProcessedDeployProto]: ...

    @property
    def systemDeploys(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[ProcessedSystemDeployProto]: ...

    def __init__(self,
        *,
        state : typing___Optional[RChainStateProto] = None,
        deploys : typing___Optional[typing___Iterable[ProcessedDeployProto]] = None,
        systemDeploys : typing___Optional[typing___Iterable[ProcessedSystemDeployProto]] = None,
        extraBytes : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BodyProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BodyProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"state",b"state"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"deploys",b"deploys",u"extraBytes",b"extraBytes",u"state",b"state",u"systemDeploys",b"systemDeploys"]) -> None: ...

class JustificationProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    validator = ... # type: builtin___bytes
    latestBlockHash = ... # type: builtin___bytes

    def __init__(self,
        *,
        validator : typing___Optional[builtin___bytes] = None,
        latestBlockHash : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> JustificationProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> JustificationProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"latestBlockHash",b"latestBlockHash",u"validator",b"validator"]) -> None: ...

class RChainStateProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    preStateHash = ... # type: builtin___bytes
    postStateHash = ... # type: builtin___bytes
    blockNumber = ... # type: builtin___int

    @property
    def bonds(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[BondProto]: ...

    def __init__(self,
        *,
        preStateHash : typing___Optional[builtin___bytes] = None,
        postStateHash : typing___Optional[builtin___bytes] = None,
        bonds : typing___Optional[typing___Iterable[BondProto]] = None,
        blockNumber : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> RChainStateProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> RChainStateProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"blockNumber",b"blockNumber",u"bonds",b"bonds",u"postStateHash",b"postStateHash",u"preStateHash",b"preStateHash"]) -> None: ...

class EventProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def produce(self) -> ProduceEventProto: ...

    @property
    def consume(self) -> ConsumeEventProto: ...

    @property
    def comm(self) -> CommEventProto: ...

    def __init__(self,
        *,
        produce : typing___Optional[ProduceEventProto] = None,
        consume : typing___Optional[ConsumeEventProto] = None,
        comm : typing___Optional[CommEventProto] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EventProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EventProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"comm",b"comm",u"consume",b"consume",u"event_instance",b"event_instance",u"produce",b"produce"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"comm",b"comm",u"consume",b"consume",u"event_instance",b"event_instance",u"produce",b"produce"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"event_instance",b"event_instance"]) -> typing_extensions___Literal["produce","consume","comm"]: ...

class ProduceEventProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    channelsHash = ... # type: builtin___bytes
    hash = ... # type: builtin___bytes
    persistent = ... # type: builtin___bool
    timesRepeated = ... # type: builtin___int

    def __init__(self,
        *,
        channelsHash : typing___Optional[builtin___bytes] = None,
        hash : typing___Optional[builtin___bytes] = None,
        persistent : typing___Optional[builtin___bool] = None,
        timesRepeated : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ProduceEventProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ProduceEventProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"channelsHash",b"channelsHash",u"hash",b"hash",u"persistent",b"persistent",u"timesRepeated",b"timesRepeated"]) -> None: ...

class ConsumeEventProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    channelsHashes = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[builtin___bytes]
    hash = ... # type: builtin___bytes
    persistent = ... # type: builtin___bool

    def __init__(self,
        *,
        channelsHashes : typing___Optional[typing___Iterable[builtin___bytes]] = None,
        hash : typing___Optional[builtin___bytes] = None,
        persistent : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ConsumeEventProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ConsumeEventProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"channelsHashes",b"channelsHashes",u"hash",b"hash",u"persistent",b"persistent"]) -> None: ...

class CommEventProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def consume(self) -> ConsumeEventProto: ...

    @property
    def produces(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[ProduceEventProto]: ...

    @property
    def peeks(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[PeekProto]: ...

    def __init__(self,
        *,
        consume : typing___Optional[ConsumeEventProto] = None,
        produces : typing___Optional[typing___Iterable[ProduceEventProto]] = None,
        peeks : typing___Optional[typing___Iterable[PeekProto]] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> CommEventProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> CommEventProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"consume",b"consume"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"consume",b"consume",u"peeks",b"peeks",u"produces",b"produces"]) -> None: ...

class PeekProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    channelIndex = ... # type: builtin___int

    def __init__(self,
        *,
        channelIndex : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> PeekProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> PeekProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"channelIndex",b"channelIndex"]) -> None: ...

class BondProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    validator = ... # type: builtin___bytes
    stake = ... # type: builtin___int

    def __init__(self,
        *,
        validator : typing___Optional[builtin___bytes] = None,
        stake : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BondProto: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BondProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"stake",b"stake",u"validator",b"validator"]) -> None: ...
